from plain.views import View

from .models import Pageview


class TrackView(View):
    def post(self) -> int:
        data = self.request.json_data

        try:
            pageview = Pageview.create_from_request(
                self.request,
                url=data["url"],
                title=data["title"],
                referrer=data["referrer"],
                timestamp=data["timestamp"],
            )
        except KeyError:
            return 400

        if pageview is None:
            return 200

        return 201



# Generated by Plain 0.68.0 on 2025-09-25 19:32

from typing import Any

from plain import models
from plain.models import migrations


def extract_tracking_params_migration(url: str) -> tuple[str, str, str]:
    """
    Extract tracking parameters from a URL for migration purposes.
    This is a copy of the logic from params.py to avoid import dependencies.
    """
    from urllib.parse import parse_qs, urlparse

    parsed = urlparse(url)
    params = parse_qs(parsed.query)

    source = ""
    medium = ""
    campaign = ""

    # Extract source (priority order)
    if utm_source := params.get("utm_source", [""])[0]:
        source = utm_source.strip().lower()
    elif ref := params.get("ref", [""])[0]:
        source = ref.strip().lower()
    elif "gclid" in params:
        source = "google"
    elif "fbclid" in params:
        source = "facebook"
    elif "msclkid" in params:
        source = "bing"
    elif "ttclid" in params:
        source = "tiktok"
    elif "twclid" in params:
        source = "twitter"

    # Extract medium
    if utm_medium := params.get("utm_medium", [""])[0]:
        medium = utm_medium.strip().lower()
    elif "gclid" in params:
        medium = "cpc"
    elif "fbclid" in params:
        medium = "social"
    elif "msclkid" in params:
        medium = "cpc"
    elif "ttclid" in params:
        medium = "cpc"
    elif "twclid" in params:
        medium = "cpc"

    # Extract campaign
    if utm_campaign := params.get("utm_campaign", [""])[0]:
        campaign = utm_campaign.strip().lower()

    return source, medium, campaign


def populate_attribution_fields(apps: Any, schema_editor: Any) -> None:
    """
    Populate source, medium, campaign fields for existing pageviews
    that have tracking parameters in their URLs.
    """
    Pageview = apps.get_model("plainpageviews", "pageview")

    # Find pageviews that have utm_source parameters
    pageviews = Pageview.query.filter(url__icontains="utm_source=")

    for pageview in pageviews:
        try:
            source, medium, campaign = extract_tracking_params_migration(pageview.url)

            if source or medium or campaign:
                pageview.source = source
                pageview.medium = medium
                pageview.campaign = campaign
                pageview.save(update_fields=["source", "medium", "campaign"])
        except Exception:
            # Skip any URLs that can't be parsed
            continue


class Migration(migrations.Migration):
    dependencies = [
        (
            "plainpageviews",
            "0002_remove_pageview_plainpagevi_session_5545cf_idx_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="pageview",
            name="campaign",
            field=models.CharField(max_length=200, required=False),
        ),
        migrations.AddField(
            model_name="pageview",
            name="medium",
            field=models.CharField(max_length=200, required=False),
        ),
        migrations.AddField(
            model_name="pageview",
            name="source",
            field=models.CharField(max_length=200, required=False),
        ),
        migrations.AddIndex(
            model_name="pageview",
            index=models.Index(fields=["source"], name="plainpagevi_source_23e80d_idx"),
        ),
        migrations.AddIndex(
            model_name="pageview",
            index=models.Index(fields=["medium"], name="plainpagevi_medium_370c1b_idx"),
        ),
        migrations.RunPython(
            populate_attribution_fields,
        ),
    ]



from datetime import timedelta

PAGEVIEWS_ASSOCIATE_ANONYMOUS_SESSIONS: bool = True
PAGEVIEWS_ANONYMOUS_RETENTION_TIMEDELTA: timedelta = timedelta(days=90)
PAGEVIEWS_AUTHENTICATED_RETENTION_TIMEDELTA: timedelta = timedelta(days=365)



from plain.admin.cards import Card, TrendCard
from plain.admin.views import (
    AdminModelDetailView,
    AdminModelListView,
    AdminViewset,
    register_viewset,
)

from .models import Pageview


class PageviewsTrendCard(TrendCard):
    title = "Pageviews trend"
    model = Pageview
    datetime_field = "timestamp"
    size = TrendCard.Sizes.FULL


@register_viewset
class PageviewAdmin(AdminViewset):
    class ListView(AdminModelListView):
        model = Pageview
        nav_section = "Pageviews"
        nav_icon = "eye"
        title = "Pageviews"
        description = "Page visits with UTM tracking and session attribution."
        fields = ["user_id", "url", "source", "timestamp", "session_id"]
        search_fields = ["id", "user_id", "url", "session_id", "source", "medium"]
        cards = [PageviewsTrendCard]

    class DetailView(AdminModelDetailView):
        model = Pageview


class UserPageviewsCard(Card):
    title = "Recent pageviews"
    template_name = "pageviews/card.html"

    def get_template_context(self) -> dict:
        context = super().get_template_context()

        # self.view has an object attribute when used in DetailView context
        context["pageviews"] = Pageview.query.filter(
            user_id=self.view.object.id  # type: ignore[attr-defined]
        ).order_by("-timestamp")[:50]

        return context



# Generated by Plain 0.68.1 on 2025-09-26 18:29

from plain.models import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "plainpageviews",
            "0003_pageview_campaign_pageview_medium_pageview_source_and_more",
        ),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name="pageview",
            name="plainpageviews_pageview_unique_uuid",
        ),
        migrations.RemoveField(
            model_name="pageview",
            name="uuid",
        ),
    ]



from __future__ import annotations

from urllib.parse import parse_qs, urlparse


def extract_tracking_params(url: str) -> tuple[str, str, str]:
    """
    Extract tracking parameters from a URL.

    Supports:
    - UTM parameters (utm_source, utm_medium, utm_campaign)
    - Simple ref parameter
    - Auto-detection of tracking IDs (gclid, fbclid)

    Args:
        url: Full URL to parse

    Returns:
        Tuple of (source, medium, campaign) strings
    """
    parsed = urlparse(url)
    params = parse_qs(parsed.query)

    source = ""
    medium = ""
    campaign = ""

    # Extract source (priority order)
    if utm_source := params.get("utm_source", [""])[0]:
        source = utm_source.strip().lower()
    elif ref := params.get("ref", [""])[0]:
        source = ref.strip().lower()
    elif "gclid" in params:
        source = "google"
    elif "fbclid" in params:
        source = "facebook"
    elif "msclkid" in params:
        source = "bing"
    elif "ttclid" in params:
        source = "tiktok"
    elif "twclid" in params:
        source = "twitter"

    # Extract medium
    if utm_medium := params.get("utm_medium", [""])[0]:
        medium = utm_medium.strip().lower()
    elif "gclid" in params:
        medium = "cpc"
    elif "fbclid" in params:
        medium = "social"
    elif "msclkid" in params:
        medium = "cpc"
    elif "ttclid" in params:
        medium = "cpc"
    elif "twclid" in params:
        medium = "cpc"

    # Extract campaign
    if utm_campaign := params.get("utm_campaign", [""])[0]:
        campaign = utm_campaign.strip().lower()

    return source, medium, campaign



from plain.chores import Chore, register_chore
from plain.runtime import settings
from plain.utils import timezone

from .models import Pageview


@register_chore
class ClearOldPageviews(Chore):
    """Delete old anonymous and authenticated pageviews."""

    def run(self) -> str:
        cutoff = timezone.now() - settings.PAGEVIEWS_ANONYMOUS_RETENTION_TIMEDELTA
        result = Pageview.query.filter(timestamp__lt=cutoff, user_id="").delete()
        output = f"{result[0]} anonymous pageviews deleted"

        cutoff = timezone.now() - settings.PAGEVIEWS_AUTHENTICATED_RETENTION_TIMEDELTA
        result = (
            Pageview.query.filter(timestamp__lt=cutoff).exclude(user_id="").delete()
        )
        output += f", {result[0]} authenticated pageviews deleted"

        return output



# Generated by Plain 0.62.1 on 2025-09-10 02:26

from plain import models
from plain.models import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainpageviews", "0001_initial"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="pageview",
            name="plainpagevi_session_5545cf_idx",
        ),
        migrations.RenameField(
            model_name="pageview",
            old_name="session_key",
            new_name="session_id",
        ),
        migrations.AddIndex(
            model_name="pageview",
            index=models.Index(
                fields=["session_id"], name="plainpagevi_session_5b738a_idx"
            ),
        ),
    ]



from __future__ import annotations

from typing import Any

from jinja2.runtime import Context

from plain.templates import register_template_extension
from plain.templates.jinja.extensions import InclusionTagExtension
from plain.urls import reverse


@register_template_extension
class PageviewsJSExtension(InclusionTagExtension):
    tags = {"pageviews_js"}
    template_name = "pageviews/js.html"

    def get_context(
        self, context: Context, *args: Any, **kwargs: Any
    ) -> dict[str, Any]:
        return {
            "pageviews_track_url": reverse("pageviews:track"),
            "request": context.get("request"),
        }



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Any

from plain import models
from plain.models import types
from plain.runtime import settings
from plain.utils import timezone

from .params import extract_tracking_params

if TYPE_CHECKING:
    from plain.http import Request

try:
    from plain.auth import get_request_user
except ImportError:
    get_request_user: Any = None

try:
    from plain.sessions import get_request_session
except ImportError:
    get_request_session: Any = None

try:
    from plain.admin.impersonate import get_request_impersonator
except ImportError:
    get_request_impersonator: Any = None


@models.register_model
class Pageview(models.Model):
    # A full URL can be thousands of characters, but MySQL has a 3072-byte limit
    # on indexed columns (when using the default ``utf8mb4`` character set that
    # stores up to 4 bytes per character). The ``url`` field is indexed below,
    # so we keep the length at 768 characters (768 × 4 = 3072 bytes) to ensure
    # the index can be created on all supported database backends.
    url: str = types.URLField(max_length=768)
    timestamp: datetime = types.DateTimeField(auto_now_add=True)

    title: str = types.CharField(max_length=512, required=False)
    # Referrers may not always be valid URLs (e.g. `android-app://...`).
    # Use a plain CharField so we don't validate the scheme or format.
    referrer: str = types.CharField(max_length=1024, required=False)

    user_id: str = types.CharField(max_length=255, required=False)
    session_id: str = types.CharField(max_length=255, required=False)

    # Attribution tracking
    source: str = types.CharField(max_length=200, required=False)
    medium: str = types.CharField(max_length=200, required=False)
    campaign: str = types.CharField(max_length=200, required=False)

    query: models.QuerySet[Pageview] = models.QuerySet()

    model_options = models.Options(
        ordering=["-timestamp"],
        indexes=[
            models.Index(fields=["timestamp"]),
            models.Index(fields=["user_id"]),
            models.Index(fields=["session_id"]),
            models.Index(fields=["url"]),
            models.Index(fields=["source"]),
            models.Index(fields=["medium"]),
        ],
    )

    def __str__(self) -> str:
        return self.url

    @classmethod
    def create_from_request(
        cls,
        request: Request,
        *,
        url: str | None = None,
        title: str | None = None,
        referrer: str | None = None,
        timestamp: datetime | None = None,
        source: str | None = None,
        medium: str | None = None,
        campaign: str | None = None,
    ) -> Pageview | None:
        """Create a pageview from a request object.

        Args:
            request: The HTTP request object
            url: Page URL (defaults to request.build_absolute_uri())
            title: Page title (defaults to empty string)
            referrer: Referring URL (defaults to Referer header)
            timestamp: Page visit time (defaults to current server time)
            source: Traffic source (auto-extracted from URL if not provided)
            medium: Traffic medium (auto-extracted from URL if not provided)
            campaign: Campaign name (auto-extracted from URL if not provided)

        Returns:
            Pageview instance or None if user is being impersonated
        """
        if get_request_impersonator and get_request_impersonator(request):
            return None

        if url is None:
            url = request.build_absolute_uri()

        if title is None:
            title = ""

        if referrer is None:
            referrer = request.headers.get("Referer", "")

        if timestamp is None:
            timestamp = timezone.now()

        # Extract tracking parameters if not provided
        if source is None or medium is None or campaign is None:
            extracted_source, extracted_medium, extracted_campaign = (
                extract_tracking_params(url)
            )
            if source is None:
                source = extracted_source
            if medium is None:
                medium = extracted_medium
            if campaign is None:
                campaign = extracted_campaign

        user = get_request_user(request) if get_request_user else None
        user_id = user.id if user else ""

        if get_request_session:
            session = get_request_session(request)
        else:
            session = None

        if session:
            session_instance = session.model_instance
            session_id = str(session_instance.id) if session_instance else ""

            if settings.PAGEVIEWS_ASSOCIATE_ANONYMOUS_SESSIONS:
                if not user_id:
                    if not session_id:
                        # Make sure we have a key to use
                        session.create()
                        session_instance = session.model_instance
                        session_id = (
                            str(session_instance.id) if session_instance else ""
                        )

                    # The user hasn't logged in yet but might later. When they do log in,
                    # the session key itself will be cycled (session fixation attacks),
                    # so we'll store the anonymous session id in the data which will be preserved
                    # when the key cycles, then remove it immediately after.
                    session["pageviews_anonymous_session_id"] = session_id
                elif user_id and "pageviews_anonymous_session_id" in session:
                    # Associate the previously anonymous pageviews with the user
                    cls.query.filter(
                        user_id="",
                        session_id=session["pageviews_anonymous_session_id"],
                    ).update(user_id=user_id)

                    # Remove it so we don't keep trying to associate it
                    del session["pageviews_anonymous_session_id"]
        else:
            session_id = ""

        return cls.query.create(
            user_id=user_id,
            session_id=session_id,
            url=url,
            title=title,
            referrer=referrer,
            timestamp=timestamp,
            source=source,
            medium=medium,
            campaign=campaign,
        )



# Generated by Plain 0.52.2 on 2025-07-08 01:17

import uuid

from plain import models
from plain.models import migrations


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Pageview",
            fields=[
                ("id", models.PrimaryKeyField()),
                ("uuid", models.UUIDField(default=uuid.uuid4)),
                ("url", models.URLField(max_length=768)),
                ("timestamp", models.DateTimeField(auto_now_add=True)),
                ("title", models.CharField(max_length=512, required=False)),
                ("referrer", models.CharField(max_length=1024, required=False)),
                ("user_id", models.CharField(max_length=255, required=False)),
                ("session_key", models.CharField(max_length=255, required=False)),
            ],
            options={
                "ordering": ["-timestamp"],
            },
        ),
        migrations.AddIndex(
            model_name="pageview",
            index=models.Index(
                fields=["timestamp"], name="plainpagevi_timesta_da4eb2_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="pageview",
            index=models.Index(
                fields=["user_id"], name="plainpagevi_user_id_b40ca5_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="pageview",
            index=models.Index(
                fields=["session_key"], name="plainpagevi_session_5545cf_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="pageview",
            index=models.Index(fields=["url"], name="plainpagevi_url_d3e821_idx"),
        ),
        migrations.AddConstraint(
            model_name="pageview",
            constraint=models.UniqueConstraint(
                fields=("uuid",), name="plainpageviews_pageview_unique_uuid"
            ),
        ),
    ]
